from os.path import basename, dirname, isfile, join, sep
from typing import Any, Dict, List, Optional, Union
from glyphspkg.filenames import userNameToFileName
from glyphspkg.plist import parse_plist_from_path, save_to_plist_path


def package_to_single(
    input_path: str, output_path: Optional[str] = None
) -> None:
    # The main dict
    input_path = input_path.rstrip(sep)
    glyphs_file = convert_fontinfo(input_path)

    # Glyph order, is used to read individual glyphs files
    glyph_order = convert_order(input_path)

    # All glyphs files derived from the Glyph order list
    glyphs = []
    for glyph_name in glyph_order:
        file_name = userNameToFileName(glyph_name)
        file_path = join(input_path, "glyphs", f"{file_name}.glyph")
        if not isfile(file_path):
            print(
                f"WARNING: Glyph file not found for glyph '{glyph_name}': "
                f"{file_name}, glyph will be missing in converted file."
            )
            continue

        glyph = parse_plist_from_path(file_path)
        glyphs.append(glyph)

    glyphs_file["glyphs"] = glyphs

    # UIState, current display strings
    uistate = convert_uistate(input_path)
    if uistate:
        # Why the different key casing?
        glyphs_file["DisplayStrings"] = uistate["displayStrings"]

    file_name = basename(input_path)
    if "." in file_name:
        file_name = file_name.rsplit(".", 1)[0]
    file_name += ".glyphs"

    if output_path is None:
        output_path = dirname(input_path)

    output_file_path = join(output_path, file_name)
    assert input_path != output_file_path
    print(f"Saving: {output_file_path}")
    save_to_plist_path(glyphs_file, output_file_path)


def convert_fontinfo(input_path: str) -> Union[Dict[Any, Any], List[Any]]:
    fontinfo_path = join(input_path, "fontinfo.plist")
    return parse_plist_from_path(fontinfo_path)


def convert_order(input_path: str) -> Union[Dict[Any, Any], List[Any]]:
    order_path = join(input_path, "order.plist")
    return parse_plist_from_path(order_path)


def convert_uistate(input_path: str) -> Union[Dict[Any, Any], List[Any]]:
    uistate_path = join(input_path, "UIState.plist")
    if not isfile(uistate_path):
        return {}

    return parse_plist_from_path(uistate_path)



"""
Command line interface for compilation/decompilation
"""

import argparse
from os.path import isdir
from glyphspkg.package_to_single import package_to_single
from glyphspkg.single_to_package import single_to_package


def main() -> None:
    parser = argparse.ArgumentParser()

    # parser.add_argument(
    #     "-i",
    #     "--in-place",
    #     action="store_true",
    #     default=False,
    #     help="Convert the file in place, i.e. remove the input file.",
    # )
    parser.add_argument(
        "-o",
        "--output",
        dest="output_path",
        type=str,
        help=(
            "Output path for converted files. If omitted, the file is saved "
            "next to the input file."
        ),
    )
    parser.add_argument(
        "glyphsfile",
        type=str,
        nargs="+",
        help="Path to the .glyphs file or .glyphspackage dir to be converted.",
    )

    arguments = parser.parse_args()

    if not arguments:
        parser.print_help()
        return

    for input_path in arguments.glyphsfile:
        print("Reading:", input_path)
        if isdir(input_path):
            # The file is a package, convert to single
            package_to_single(input_path, arguments.output_path)
        else:
            # The file is a single file, convert to package
            single_to_package(input_path, arguments.output_path)



from setuptools import setup


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="glyphspkg",
    description="Converter from .glyphspackage to .glyphs files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Jens Kutilek",
    url="https://github.com/jenskutilek/glyphspkg",
    packages=[
        "glyphspkg",
    ],
    package_dir={"": "Lib"},
    entry_points={
        "console_scripts": [
            "glyphspkg = glyphspkg.cmdline:main",
        ]
    },
    install_requires=[
        "openstep-plist >= 0.3.0",
    ],
    setup_requires=[
        "setuptools_scm",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Multimedia :: Graphics :: Graphics Conversion",
        "Topic :: Multimedia :: Graphics :: Editors :: Vector-Based",
    ],
)
